package wusthelper.web.api.v2.interceptor;

import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.annotation.Nonnull;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;
import wusthelper.web.annotation.RequireToken;
import wusthelper.web.api.v2.TokenTool;
import wusthelper.web.api.v2.code.V2ResponseCode;
import wusthelper.web.response.Response;

import java.io.IOException;

@Slf4j
@Component
public class TokenInterceptor implements HandlerInterceptor {

    private final ObjectMapper objectMapper;

    public TokenInterceptor(ObjectMapper objectMapper) {
        this.objectMapper = objectMapper;
    }

    @Override
    public boolean preHandle(@Nonnull HttpServletRequest request,
                             @Nonnull HttpServletResponse response,
                             @Nonnull Object handler) throws Exception {

        if (!(handler instanceof HandlerMethod handlerMethod)) {
            return true;
        }

        // 方法上的RequireToken
        var methodAnnotation = handlerMethod.getMethodAnnotation(RequireToken.class);
        if (methodAnnotation == null) {
            // class上的RequireToken
            var classAnnotation = handlerMethod.getMethod().getDeclaringClass().getAnnotation(RequireToken.class);
            if (classAnnotation == null) {
                return true;
            } else {
                if (!classAnnotation.require()) {
                    return true;
                }
            }
        } else {
            if (!methodAnnotation.require()) {
                return true;
            }
        }

        // 验证token
        var token = request.getHeader("token");
        if (token == null || "".equals(token)) {
            this.rejectRequest(response);
            return false;
        }

        var tokenValid = verifyToken(token);
        if (!tokenValid) {
            this.rejectRequest(response);
            return false;
        }

        return true;
    }

    private boolean verifyToken(String token) {
        return TokenTool.verify(token);
    }

    private void rejectRequest(HttpServletResponse servletResponse) throws IOException {
        var response = Response.error(V2ResponseCode.AUTH_ERR_TOKEN_INVALID.getCode(), "token无效或已过期，请重新登录");
        var json = objectMapper.writeValueAsString(response);

        servletResponse.setContentType("application/json");
        servletResponse.setCharacterEncoding("utf-8");
        servletResponse.getWriter().write(json);
    }
}
