package wusthelper.web.service.campus;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.yitter.idgen.YitIdHelper;
import org.springframework.stereotype.Service;
import wusthelper.data.dao.mapper.UserBasicMapper;
import wusthelper.data.entity.UserBasic;
import wusthelper.web.data.entity.CookieType;
import wusthelper.web.service.campus.graduate.GraduateCookieService;
import wusthelper.web.service.campus.library.LibraryCookieService;
import wusthelper.web.service.campus.physics.PhysicsCookieService;
import wusthelper.web.service.campus.undergrad.UndergradCookieService;
import wusthelper.web.service.cookie.CookieManager;
import wusthelper.web.util.PasswordCodec;

/**
 * 通用的用户登录信息服务，包括登录和（已存储的）用户信息获取等，同时自动存入新用户密码信息
 */
@Service
public class UserLoginService {
    private final UserBasicMapper userBasicMapper;
    private final PasswordCodec passwordCodec;

    private final UndergradCookieService undergradCookieService;
    private final GraduateCookieService graduateCookieService;
    private final LibraryCookieService libraryCookieService;
    private final PhysicsCookieService physicsCookieService;

    public UserLoginService(UserBasicMapper userBasicMapper,
                            PasswordCodec passwordCodec,

                            UndergradCookieService undergradCookieService,
                            GraduateCookieService graduateCookieService,
                            LibraryCookieService libraryCookieService,
                            PhysicsCookieService physicsCookieService) {

        this.userBasicMapper = userBasicMapper;
        this.passwordCodec = passwordCodec;

        this.undergradCookieService = undergradCookieService;
        this.graduateCookieService = graduateCookieService;
        this.libraryCookieService = libraryCookieService;
        this.physicsCookieService = physicsCookieService;
    }

    public enum LoginType {Undergrad, Graduate, Library, Physics}

    /**
     * 登录验证，同时自动存库（基本信息）
     *
     * @param username  用户名（即学号）
     * @param password  密码（明文原文）
     * @param loginType 用户类型，现在有本科生和研究生
     * @return 用户信息
     */
    public UserBasic login(String username, String password, LoginType loginType) {
        var cookie = switch (loginType) {
            case Undergrad -> undergradCookieService.getLoginCookie(username, password);
            case Graduate -> graduateCookieService.getLoginCookie(username, password);
            case Library -> libraryCookieService.getLoginCookie(username, password);
            case Physics -> physicsCookieService.getLoginCookie(username, password);
        };

        var cookieType = switch (loginType) {
            case Undergrad -> CookieType.Undergrad;
            case Graduate -> CookieType.Graduate;
            case Library -> CookieType.Library;
            case Physics -> CookieType.Physics;
        };

        // 保存cookie，并将用户基本信息存库
        CookieManager.store(username, cookie, cookieType);
        return this.saveOrUpdateUser(username, password, loginType);
    }

    private UserBasic saveOrUpdateUser(String username, String password, LoginType loginType) {
        var user = userBasicMapper.selectOne(new QueryWrapper<UserBasic>()
                .eq("stu_num", username)
                .eq("deleted", 0)
                .last("limit 1")
        );

        if (user != null) {
            this.fillPassword(password, user, loginType);
            this.updateUser(username, user);
        } else {
            user = this.newUser(username, password, loginType);
        }

        return user;
    }

    private void updateUser(String username, UserBasic user) {
        userBasicMapper.update(user, new QueryWrapper<UserBasic>()
                .eq("stu_num", username)
                .eq("deleted", 0)
                .last("limit 1")
        );
    }

    private UserBasic newUser(String username, String password, LoginType loginType) {
        var uid = YitIdHelper.nextId();
        UserBasic user = UserBasic.builder()
                .uid(uid)
                .stuNum(username)
                .build();

        this.fillPassword(password, user, loginType);

        userBasicMapper.insert(user);
        return user;
    }

    private void fillPassword(String password, UserBasic user, LoginType loginType) {
        var encrypted = passwordCodec.encode(password);
        switch (loginType) {
            case Undergrad -> {
                user.setOfficialPwd(encrypted);
                user.setLibPwd(encrypted);
            }
            case Graduate -> user.setOfficialPwd(encrypted);
            case Library -> user.setLibPwd(encrypted);
            case Physics -> user.setPhysicsPwd(encrypted);
        }
    }
}
