package wusthelper.web.service.cookie;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import wusthelper.code.ServiceCode;
import wusthelper.data.dao.mapper.UserBasicMapper;
import wusthelper.data.entity.UserBasic;
import wusthelper.web.data.entity.CookieType;
import wusthelper.web.exception.ServiceException;
import wusthelper.web.service.campus.graduate.GraduateCookieService;
import wusthelper.web.service.campus.library.LibraryCookieService;
import wusthelper.web.service.campus.physics.PhysicsCookieService;
import wusthelper.web.service.campus.undergrad.UndergradCookieService;
import com.github.benmanes.caffeine.cache.Cache;
import com.github.benmanes.caffeine.cache.Caffeine;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Service;
import wusthelper.web.util.PasswordCodec;

import java.util.concurrent.TimeUnit;

@Service
@Scope("singleton")
public class CookieManager {
    private static final Cache<String, String> cookiePool = Caffeine.newBuilder()
            .expireAfterWrite(2, TimeUnit.HOURS)
            .initialCapacity(256)
            .maximumSize(1024)
            .build();

    private final UndergradCookieService undergradCookieService;
    private final GraduateCookieService graduateCookieService;
    private final LibraryCookieService libraryCookieService;
    private final PhysicsCookieService physicsCookieService;
    private final PasswordCodec passwordCodec;

    private final UserBasicMapper studentMapper;

    public CookieManager(UndergradCookieService undergradCookieService,
                         GraduateCookieService graduateCookieService,
                         LibraryCookieService libraryCookieService,
                         PhysicsCookieService physicsCookieService,
                         PasswordCodec passwordCodec,

                         UserBasicMapper userBasicMapper) {

        this.undergradCookieService = undergradCookieService;
        this.graduateCookieService = graduateCookieService;
        this.libraryCookieService = libraryCookieService;
        this.physicsCookieService = physicsCookieService;
        this.passwordCodec = passwordCodec;

        this.studentMapper = userBasicMapper;
    }

    public String getCookie(String username, CookieType cookieType) {
        return getCookie(username, cookieType, false);
    }

    public String getCookie(String username, CookieType cookieType, boolean forceRefresh) {
        String cacheKey = String.format("%s:%d", username, cookieType.getCode());
        String cookie = cookiePool.getIfPresent(cacheKey);

        boolean valid = cookie != null && checkCookie(cookie, cookieType) && !forceRefresh;
        if (valid) {
            return cookie;
        }

        cookie = this.refreshCookie(username, cookieType);
        cookiePool.put(cacheKey, cookie);

        return cookie;
    }

    public static void store(String username, String cookie, CookieType cookieType) {
        String cacheKey = String.format("%s:%d", username, cookieType.getCode());
        cookiePool.put(cacheKey, cookie);
    }

    public boolean checkCookie(String cookie, CookieType cookieType) {
        return switch (cookieType) {
            case Undergrad -> undergradCookieService.checkCookie(cookie);
            case Graduate -> graduateCookieService.checkCookie(cookie);
            case Library -> libraryCookieService.checkCookie(cookie);
            case Physics -> physicsCookieService.checkCookie(cookie);
        };
    }

    public String refreshCookie(String username, CookieType cookieType) {
        var student = studentMapper.selectOne(new QueryWrapper<UserBasic>()
                .eq("stu_num", username)
                .eq("deleted", 0)
        );

        if (student == null) {
            ServiceException.error(ServiceCode.UserNotExists);
        }

        String password = switch (cookieType) {
            case Undergrad, Graduate -> passwordCodec.decode(student.getOfficialPwd());
            case Library -> passwordCodec.decode(student.getLibPwd());
            case Physics -> passwordCodec.decode(student.getPhysicsPwd());
        };

        if (password == null) {
            ServiceException.error(ServiceCode.ParamWrong);
        }

        return switch (cookieType) {
            case Undergrad -> undergradCookieService.getLoginCookie(username, password);
            case Graduate -> graduateCookieService.getLoginCookie(username, password);
            case Library -> libraryCookieService.getLoginCookie(username, password);
            case Physics -> physicsCookieService.getLoginCookie(username, password);
        };
    }
}